@@ -189,12 +189,16 @@ class InferenceEngine(Module):
             if torch.is_tensor(input):
                 input = input.to(torch.cuda.current_device())
                 if self.mp_world_size > 1:
+                    if not input.is_contiguous():
+                        input = input.contiguous()
                     dist.broadcast(input, 0)
 
         for k in kwargs:
             if torch.is_tensor(kwargs[k]):
                 kwargs[k] = kwargs[k].to(torch.cuda.current_device())
                 if self.mp_world_size > 1:
+                    if not kwargs[k].is_contiguous():
+                        kwargs[k] = kwargs[k].contiguous()
                     dist.broadcast(kwargs[k], 0)
 
     def forward(self, *inputs, **kwargs):
@@ -210,12 +214,16 @@ class InferenceEngine(Module):
                     if torch.is_tensor(input):
                         input = input.to(torch.cuda.current_device())
                         if self.mp_world_size > 1:
+                            if not input.is_contiguous():
+                                input = input.contiguous()
                             dist.broadcast(input, 0)
 
                 for k in kwargs:
                     if torch.is_tensor(kwargs[k]):
                         kwargs[k] = kwargs[k].to(torch.cuda.current_device())
                         if self.mp_world_size > 1:
+                            if not kwargs[k].is_contiguous():
+                                kwargs[k] = kwargs[k].contiguous()
                             dist.broadcast(kwargs[k], 0)
 
             return self.model_orig_fwd(*inputs, **kwargs)
